<?php

namespace Drupal\entity_copy_reference\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\entity_copy_reference\EntityCopyReference;
use Drupal\node\Entity\Node;

/**
 * Class EntityCopyReferenceConfirm.
 *
 * @package Drupal\entity_copy_reference\Form
 */
class EntityCopyReferenceConfirm extends FormBase implements FormInterface {

  /**
   * Step variable (optional use).
   *
   * @var step
   */
  protected $step = 1;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'entity_copy_reference_confirm';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#attached']['library'][] = 'entity_copy_reference/entity-copy-reference-form';

    $form['#title'] = $this->t('Entity Copy with Reference');

    $form['subtitle'] = [
      '#prefix' => '<h4>',
      '#markup' => '',
      '#suffix' => '</h4>',
    ];

    $node = \Drupal::routeMatch()->getParameter('node');
    $node = Node::load($node);
    // Check if this is a node of the types you want to handle.
    $form['subtitle']['#markup'] = $this->t('Create copy of node <em>@node</em>?', ['@node' => $node->label()]);

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Create'),
      '#button_type' => 'primary',
    ];

    $edit_link = Url::fromRoute('entity.node.edit_form', ['node' => $node->id()])->toString();
    $form['cancel'] = [
      '#prefix' => '<a href="' . $edit_link . '" class="button">',
      '#markup' => $this->t('Cancel'),
      '#suffix' => '</a>',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $node = \Drupal::routeMatch()->getParameter('node');
    if (!is_object($node)) {
      $node = Node::load($node);
    }

    $entity_copy_reference = new EntityCopyReference();
    $clone = $entity_copy_reference->copyEntity($node);

    $url = Url::fromRoute('entity_copy_reference.copy', ['node' => $node->id()]);

    if ($clone) {
      $current_user = \Drupal::currentUser();
      $cloned_node = Node::load($clone['target_id']);

      // Set the field_cloned_by to current user, if the field exists.
      if ($cloned_node && $cloned_node->hasField('field_cloned_by')) {
        $cloned_node->set('field_cloned_by', $current_user->id());
        $cloned_node->save();
      }

      // Success message.
      \Drupal::messenger()->addStatus($this->t('The entity has been copied successfully.'));

      // Log user who cloned.
      \Drupal::logger('entity_copy_reference')->notice(
        'The node @node has been copied by @user.',
        [
          '@node' => $node->label(),
          '@user' => $current_user->getDisplayName(),
        ]
      );

      // Redirect to the new cloned node.
      $url = Url::fromRoute('entity.node.canonical', ['node' => $clone['target_id']]);

    } else {
      \Drupal::messenger()->addError($this->t('An error occurred while copying node @node', ['@node' => $node->label()]));
      \Drupal::logger('entity_copy_reference')->error('There was an error copying node @node', ['@node' => $node->label()]);
    }

    $form_state->setRedirectUrl($url);
  }

}
